@@ -9,6 +9,7 @@ except ImportError:  # python < 3.8
 
 from . import caching
 from ._version import get_versions
+from .callbacks import Callback
 from .core import get_fs_token_paths, open, open_files, open_local
 from .exceptions import FSTimeoutError
 from .mapping import FSMap, get_mapper
@@ -38,6 +39,7 @@ __all__ = [
     "open_local",
     "registry",
     "caching",
+    "Callback",
 ]
 
 if entry_points is not None:
