@@ -1,11 +1,13 @@
-from __future__ import print_function
+import logging
 import os
 import stat
-from errno import ENOENT, EIO
-from fuse import Operations, FuseOSError
 import threading
 import time
-from fuse import FUSE
+from errno import EIO, ENOENT
+
+from fuse import FUSE, FuseOSError, Operations
+
+logger = logging.getLogger("fsspec.fuse")
 
 
 class FUSEr(Operations):
@@ -14,8 +16,10 @@ class FUSEr(Operations):
         self.cache = {}
         self.root = path.rstrip("/") + "/"
         self.counter = 0
+        logger.info("Starting FUSE at %s", path)
 
     def getattr(self, path, fh=None):
+        logger.debug("getattr %s", path)
         path = "".join([self.root, path.lstrip("/")]).rstrip("/")
         try:
             info = self.fs.info(path)
@@ -39,6 +43,7 @@ class FUSEr(Operations):
         return data
 
     def readdir(self, path, fh):
+        logger.debug("readdir %s", path)
         path = "".join([self.root, path.lstrip("/")])
         files = self.fs.ls(path, False)
         files = [os.path.basename(f.rstrip("/")) for f in files]
@@ -55,17 +60,20 @@ class FUSEr(Operations):
         return 0
 
     def read(self, path, size, offset, fh):
+        logger.debug("read %s", (path, size, offset))
         f = self.cache[fh]
         f.seek(offset)
         out = f.read(size)
         return out
 
     def write(self, path, data, offset, fh):
+        logger.debug("read %s", (path, offset))
         f = self.cache[fh]
         f.write(data)
         return len(data)
 
     def create(self, path, flags, fi=None):
+        logger.debug("create %s", (path, flags))
         fn = "".join([self.root, path.lstrip("/")])
         f = self.fs.open(fn, "wb")
         self.cache[self.counter] = f
@@ -73,6 +81,7 @@ class FUSEr(Operations):
         return self.counter - 1
 
     def open(self, path, flags):
+        logger.debug("open %s", (path, flags))
         fn = "".join([self.root, path.lstrip("/")])
         if flags % 2 == 0:
             # read
@@ -113,7 +122,7 @@ class FUSEr(Operations):
 
 
 def run(fs, path, mount_point, foreground=True, threads=False):
-    """ Mount stuff in a local directory
+    """Mount stuff in a local directory
 
     This uses fusepy to make it appear as if a given path on an fsspec
     instance is in fact resident within the local file-system.
