@@ -572,8 +572,8 @@ class WholeFileCacheFileSystem(CachingFileSystem):
 
         out = {}
         callback.set_size(len(paths))
-        for path, fn in zip(paths, fns):
-            out[path] = open(fn, "rb").read()
+        for p, fn in zip(paths, fns):
+            out[p] = open(fn, "rb").read()
             callback.relative_update(1)
         if isinstance(path, str) and len(paths) == 1 and recursive is False:
             out = out[paths[0]]
