@@ -4,7 +4,7 @@ import os
 import pathlib
 import re
 import sys
-from hashlib import sha256
+from hashlib import md5
 from urllib.parse import urlsplit
 
 DEFAULT_BLOCK_SIZE = 5 * 2 ** 20
@@ -98,11 +98,12 @@ def update_storage_options(options, inherited=None):
         inherited = {}
     collisions = set(options) & set(inherited)
     if collisions:
-        collisions = "\n".join("- %r" % k for k in collisions)
-        raise KeyError(
-            "Collision between inferred and specified storage "
-            "options:\n%s" % collisions
-        )
+        for collision in collisions:
+            if options.get(collision) != inherited.get(collision):
+                raise KeyError(
+                    "Collision between inferred and specified storage "
+                    "option:\n%s" % collision
+                )
     options.update(inherited)
 
 
@@ -278,7 +279,7 @@ def tokenize(*args, **kwargs):
     """
     if kwargs:
         args += (kwargs,)
-    return sha256(str(args).encode()).hexdigest()
+    return md5(str(args).encode()).hexdigest()
 
 
 def stringify_path(filepath):
@@ -334,7 +335,7 @@ def common_prefix(paths):
     return "/".join(parts[0][:i])
 
 
-def other_paths(paths, path2, is_dir=None):
+def other_paths(paths, path2, is_dir=None, exists=False):
     """In bulk file operations, construct a new file tree from a list of files
 
     Parameters
@@ -348,6 +349,9 @@ def other_paths(paths, path2, is_dir=None):
         For the special case where the input in one element, whether to regard the value
         as the target path, or as a directory to put a file path within. If None, a
         directory is inferred if the path ends in '/'
+    exists: bool (optional)
+        For a str destination, it is already exists (and is a dir), files should
+        end up inside.
 
     Returns
     -------
@@ -358,6 +362,8 @@ def other_paths(paths, path2, is_dir=None):
         path2 = path2.rstrip("/")
         if len(paths) > 1:
             cp = common_prefix(paths)
+            if exists:
+                cp = cp.rsplit("/", 1)[0]
             path2 = [p.replace(cp, path2, 1) for p in paths]
         else:
             if is_dir:
